/*
 * aln_write.c
 *
 * Copyright (c) 2014 Kirill Kryukov
 *
 * This file is a part of the Alignment Mismatch Finder source code,
 * and may only be used, modified, and distributed under the terms of
 * the license, located in the file LICENSE.txt within this package.
 */

#ifdef INCLUDED_FROM_ALL_C



/*
 * Writes alignment region, delimited by 'first' and 'last' variants.
 */
void write_alignment_region(taln *aln,long long first,long long last)
{
    long long start, end, length, pos;
    /*long long n_var;*/
    /*double e;*/

    assert(aln);
    assert(aln->seq[0].data.length);
    assert(aln->seq[0].data.start);
    assert(aln->seq[1].data.length);
    assert(aln->seq[1].data.start);

    if (first < 0) { return; }

    assert(first <= last);
    assert(last <= n_vars);

    /*n_var = last - first + 1;*/

    start = vars[first].start - extend_output_regions;
    if (start < 0) { start = 0; }

    end = vars[last].end + extend_output_regions;
    if (end >= (long long)aln->seq[0].data.length) { end = aln->seq[0].data.length - 1; }

    length = end - start + 1;

    /*e = (n_var <= P_max_mismatches && length <= P_max_length) ? P[n_var*P_length_size+length] : -1.0;*/

    if (output_format == al_format_fasta)
    {
        printf(">aln%lld_seq1_%lld_%lld\n",n_alignments_looked_at_total+1,start+1,end+1);
        fwrite(aln->seq[0].data.start+start,1,length,stdout);
        putchar('\n');
        printf(">aln%lld_seq2_%lld_%lld\n",n_alignments_looked_at_total+1,start+1,end+1);
        fwrite(aln->seq[1].data.start+start,1,length,stdout);
        putchar('\n');
    }
    else if (output_format == al_format_nss)
    {
        printf("aln%lld_%lld_%lld\t",n_alignments_looked_at_total+1,start+1,end+1);
        fwrite(aln->seq[0].data.start+start,1,length,stdout);
        putchar('\t');
        fwrite(aln->seq[1].data.start+start,1,length,stdout);
        putchar('\n');
    }
    else if (output_format == al_format_cmeh)
    {
        long long n_sites_no_gap_no_mask = 0ll;  /* All aligned sites extept gaps and masked sites (N, X, optinally CG). */
        long long n_CG = 0ll, n_ACGT = 0ll;
        long long n_mismatches = 0ll;
        long long n_gap_blocks_seq_1 = 0ll;
        long long n_gap_blocks_seq_2 = 0ll;

        for (pos=start; pos<=end; pos++)
        {
            int c1 = toupper(((unsigned char*)aln->seq[0].data.start)[pos]);
            int c2 = toupper(((unsigned char*)aln->seq[1].data.start)[pos]);

            if (c1 == 'A' || c1 == 'C' || c1 == 'G' || c1 == 'T') { n_ACGT++; if (c1 == 'C' || c1 == 'G') { n_CG++; } }
            if (c2 == 'A' || c2 == 'C' || c2 == 'G' || c2 == 'T') { n_ACGT++; if (c2 == 'C' || c2 == 'G') { n_CG++; } }
            if (c1 == '-' && (pos==start || aln->seq[0].data.start[pos-1]!='-')) { n_gap_blocks_seq_1++; }
            if (c2 == '-' && (pos==start || aln->seq[1].data.start[pos-1]!='-')) { n_gap_blocks_seq_2++; }

            if (c1 == '-' || c1 == 'N' || c1 == 'X') { continue; }
            if (c2 == '-' || c2 == 'N' || c2 == 'X') { continue; }
            if (ignore_CpG)
            {
                if (c1 == 'C' && pos < (long long)aln->seq[0].data.length-1 && toupper(((unsigned char*)aln->seq[0].data.start)[pos+1]) == 'G') { continue; }
                if (c2 == 'C' && pos < (long long)aln->seq[1].data.length-1 && toupper(((unsigned char*)aln->seq[1].data.start)[pos+1]) == 'G') { continue; }
            }
            if (c1 != c2) { n_mismatches++; }
            n_sites_no_gap_no_mask++;
        }

        printf("aln%lld_%lld_%lld\t%lld\t%lld\t",n_alignments_looked_at_total+1,start+1,end+1,length,n_sites_no_gap_no_mask);
        if (n_ACGT) { printf("%.5f",(double)n_CG/n_ACGT); } else { putchar('-'); }
        printf("\t%lld\t",n_mismatches);
        if (n_sites_no_gap_no_mask)
        {
            printf("%.5f\t%.5f\t%.5f",(double)n_mismatches      /n_sites_no_gap_no_mask,
                                      (double)n_gap_blocks_seq_1/n_sites_no_gap_no_mask,
                                      (double)n_gap_blocks_seq_2/n_sites_no_gap_no_mask);
        }
        else { fputs("-\t-\t-",stdout); }
        putchar('\t');
        if (n_gap_blocks_seq_2) { printf("%.5f",(double)n_gap_blocks_seq_1/n_gap_blocks_seq_2); } else { putchar('-'); }
        putchar('\t');
        if (n_sites_no_gap_no_mask + n_gap_blocks_seq_1 + n_gap_blocks_seq_2)
        {
            printf("%.5f",(double)(n_mismatches + n_gap_blocks_seq_1 + n_gap_blocks_seq_2)/(n_sites_no_gap_no_mask + n_gap_blocks_seq_1 + n_gap_blocks_seq_2));
        }
        else { putchar('-'); }
        putchar('\t');
        fwrite(aln->seq[0].data.start+start,1,length,stdout);
        putchar('\t');
        fwrite(aln->seq[1].data.start+start,1,length,stdout);
        fputs("\t-\t-\n",stdout);
    }
    else
    {
        assert(0);

        /*int i;

        printf("%lld variants in %lld bp (%lld..%lld), E-value: ",n_var,length,start+1,end+1);
        if (e >= 0) { printf("%.3e\n",e); } else { printf("-\n"); }

        for (i=0; i<2; i++)
        {
            char c = aln->seq[i].data.start[end+1];
            aln->seq[i].data.start[end+1] = 0;
            puts(aln->seq[i].data.start+start);
            aln->seq[i].data.start[end+1] = c;
        }*/
    }

    n_regions_total++;
}



#endif
